@@ -56,12 +56,11 @@ static void __init handle_initrd(void)
 	sys_mount(".", "/", NULL, MS_MOVE, NULL);
 	sys_chroot(".");
 
-	current->flags |= PF_NOFREEZE;
 	pid = kernel_thread(do_linuxrc, "/linuxrc", SIGCHLD);
 	if (pid > 0) {
 		while (pid != sys_wait4(-1, NULL, 0, NULL)) {
-			yield();
 			try_to_freeze();
+			yield();
 		}
 	}
 
@@ -69,7 +68,6 @@ static void __init handle_initrd(void)
 		printk(KERN_ERR "Suspend2: No attempt was made to resume from "
 				"any image that might exist.\n");
 	clear_suspend_state(SUSPEND_BOOT_TIME);
-	current->flags &= ~PF_NOFREEZE;
 
 	/* move initrd to rootfs' /old */
 	sys_fchdir(old_fd);
