@@ -480,7 +480,7 @@ static void flag_image_pages(int atomic_copy)
 			else
 				page = saveable_page(pfn);
 
-			if (!page) {
+			if (!page || PageNosave(page)) {
 				num_nosave++;
 				continue;
 			}
