@@ -1,7 +1,7 @@
 /*
  * kernel/power/suspend_swap.c
  *
- * Copyright 2004-2006 Nigel Cunningham <nigel@suspend2.net>
+ * Copyright (C) 2004-2007 Nigel Cunningham (nigel at suspend2 net)
  *
  * Distributed under GPLv2.
  * 
@@ -14,6 +14,7 @@
 #include <linux/blkdev.h>
 #include <linux/swapops.h>
 #include <linux/swap.h>
+#include <linux/syscalls.h>
 
 #include "suspend.h"
 #include "sysfs.h"
@@ -23,7 +24,7 @@
 #include "extent.h"
 #include "block_io.h"
 
-static struct suspend_module_ops swapwriterops;
+static struct suspend_module_ops suspend_swapops;
 
 #define SIGNATURE_VER 6
 
@@ -51,23 +52,24 @@ static struct block_device *header_block_device;
 static unsigned long headerblock;
 
 /* For swapfile automatically swapon/off'd. */
-static char swapfilename[SWAP_FILENAME_MAXLENGTH] = "";
-extern asmlinkage long sys_swapon(const char *specialfile, int swap_flags);
-extern asmlinkage long sys_swapoff(const char *specialfile);
+static char swapfilename[32] = "";
 static int suspend_swapon_status;
 
 /* Header Page Information */
 static int header_pages_allocated;
 
+/* Swap Pages */
+static int main_pages_allocated, main_pages_requested;
+
 /* User Specified Parameters. */
 
 static unsigned long resume_firstblock;
 static int resume_blocksize;
-static dev_t resume_dev_t;
+static dev_t resume_swap_dev_t;
 static struct block_device *resume_block_device;
 
 struct sysinfo swapinfo;
-static int swapwriter_invalidate_image(void);
+static int suspend_swap_invalidate_image(void);
 
 /* Block devices open. */
 struct bdev_opened
@@ -112,7 +114,7 @@ static void close_bdevs(void)
 	resume_block_device = header_block_device = NULL;
 }
 
-static struct block_device *open_bdev(int index, dev_t device)
+static struct block_device *open_bdev(int index, dev_t device, int display_errs)
 {
 	struct bdev_opened *this;
 	struct block_device *bdev;
@@ -128,16 +130,21 @@ static struct block_device *open_bdev(int index, dev_t device)
 	bdev = open_by_devnum(device, FMODE_READ);
 
 	if (IS_ERR(bdev) || !bdev) {
-		suspend_early_boot_message(1,SUSPEND_CONTINUE_REQ,  
+		if (display_errs)
+			suspend_early_boot_message(1,SUSPEND_CONTINUE_REQ,  
 				"Failed to get access to block device "
-				"\"%s\" (error %d).\n Maybe you need "
+				"\"%x\" (error %d).\n Maybe you need "
 				"to run mknod and/or lvmsetup in an "
 				"initrd/ramfs?", device, bdev);
 		return ERR_PTR(-EINVAL);
 	}
 
 	this = kmalloc(sizeof(struct bdev_opened), GFP_KERNEL);
-	BUG_ON(!this);
+	if (!this) {
+		printk(KERN_WARNING "Suspend2: Failed to allocate memory for "
+				"opening a bdev.");
+		return ERR_PTR(-ENOMEM);
+	}
 
 	bdev_info_list[index] = this;
 	this->device = device;
@@ -163,11 +170,10 @@ static int enable_swapfile(void)
 		/* Attempt to swap on with maximum priority */
 		activateswapresult = sys_swapon(swapfilename, 0xFFFF);
 		if ((activateswapresult) && (activateswapresult != -EBUSY))
-			printk(name_suspend
-				"The swapfile/partition specified by "
-				"/sys/power/suspend2/swapwriter/swapfile (%s) could not"
-				" be turned on (error %d). Attempting "
-				"to continue.\n",
+			printk("Suspend2: The swapfile/partition specified by "
+				"/sys/power/suspend2/suspend_swap/swapfile "
+				"(%s) could not be turned on (error %d). "
+				"Attempting to continue.\n",
 				swapfilename, activateswapresult);
 		if (!activateswapresult)
 			suspend_swapon_status = 1;
@@ -194,40 +200,49 @@ static int disable_swapfile(void)
 	return result;
 }
 
-static int try_to_parse_resume_device(char *commandline)
+static int try_to_parse_resume_device(char *commandline, int quiet)
 {
 	struct kstat stat;
-	int error;
+	int error = 0;
 
-	resume_dev_t = name_to_dev_t(commandline);
+	resume_swap_dev_t = name_to_dev_t(commandline);
 
-	if (!resume_dev_t) {
-		error = vfs_stat(commandline, &stat);
+	if (!resume_swap_dev_t) {
+		struct file *file = filp_open(commandline, O_RDONLY, 0);
+
+		if (!IS_ERR(file) && file) {
+			vfs_getattr(file->f_vfsmnt, file->f_dentry, &stat);
+			filp_close(file, NULL);
+		} else
+			error = vfs_stat(commandline, &stat);
 		if (!error)
-			resume_dev_t = stat.rdev;
+			resume_swap_dev_t = stat.rdev;
 	}
 
-	if (!resume_dev_t) {
+	if (!resume_swap_dev_t) {
+		if (quiet)
+			return 1;
+
 		if (test_suspend_state(SUSPEND_TRYING_TO_RESUME))
 			suspend_early_boot_message(1, SUSPEND_CONTINUE_REQ,
 			  "Failed to translate \"%s\" into a device id.\n",
 			  commandline);
 		else
-			printk(name_suspend
-			  "Can't translate \"%s\" into a device id yet.\n",
-			  commandline);
+			printk("Suspend2: Can't translate \"%s\" into a device "
+					"id yet.\n", commandline);
 		return 1;
 	}
 
-	if (IS_ERR(resume_block_device =
-	     open_bdev(MAX_SWAPFILES, resume_dev_t))) {
-		suspend_early_boot_message(1, SUSPEND_CONTINUE_REQ,
-			"Failed to get access to \"%s\", where"
-			" the swap header should be found.",
-			commandline);
+	resume_block_device = open_bdev(MAX_SWAPFILES, resume_swap_dev_t, 0);
+	if (IS_ERR(resume_block_device)) {
+		if (!quiet)
+			suspend_early_boot_message(1, SUSPEND_CONTINUE_REQ,
+				"Failed to get access to \"%s\", where"
+				" the swap header should be found.",
+				commandline);
 		return 1;
 	}
-
+	
 	return 0;
 }
 
@@ -235,10 +250,9 @@ static int try_to_parse_resume_device(char *commandline)
  * If we have read part of the image, we might have filled  memory with
  * data that should be zeroed out.
  */
-static void swapwriter_noresume_reset(void)
+static void suspend_swap_noresume_reset(void)
 {
 	memset((char *) &devinfo, 0, sizeof(devinfo));
-	close_bdevs();
 }
 
 static int parse_signature(char *header, int restore)
@@ -328,14 +342,18 @@ static int prepare_signature(dev_t bdev, unsigned long block,
 	return 0;
 }
 
-static int swapwriter_allocate_storage(int space_requested);
+static int __suspend_swap_allocate_storage(int main_storage_requested,
+		int header_storage);
 
-static int swapwriter_allocate_header_space(int space_requested)
+static int suspend_swap_allocate_header_space(int space_requested)
 {
 	int i;
 
-	if (!swapextents.size)
-		swapwriter_allocate_storage(space_requested);
+	if (!swapextents.size && __suspend_swap_allocate_storage(
+				main_pages_requested, space_requested)) {
+		printk("Failed to allocate space for the header.\n");
+		return -ENOSPC;
+	}
 
 	suspend_extent_state_goto_start(&suspend_writer_posn);
 	suspend_bio_ops.forward_one_page(); /* To first page */
@@ -411,36 +429,37 @@ static void get_main_pool_phys_params(void)
 			extent_min, extent_max);
 	}
 
-	swapwriter_allocate_header_space(header_pages_allocated);
+	suspend_swap_allocate_header_space(header_pages_allocated);
 }
 
-static int swapwriter_storage_allocated(void)
+static int suspend_swap_storage_allocated(void)
 {
-	return swapextents.size;
+	return main_pages_requested + header_pages_allocated;
 }
 
-static int swapwriter_storage_available(void)
+static int suspend_swap_storage_available(void)
 {
 	si_swapinfo(&swapinfo);
-	return swapinfo.freeswap + swapwriter_storage_allocated();
+	return (((int) swapinfo.freeswap + main_pages_allocated) * PAGE_SIZE /
+		(PAGE_SIZE + sizeof(unsigned long) + sizeof(int)));
 }
 
-static int swapwriter_initialise(int starting_cycle)
+static int suspend_swap_initialise(int starting_cycle)
 {
 	if (!starting_cycle)
 		return 0;
 
 	enable_swapfile();
 
-	if (resume_dev_t && !resume_block_device &&
+	if (resume_swap_dev_t && !resume_block_device &&
 	    IS_ERR(resume_block_device =
-	    		open_bdev(MAX_SWAPFILES, resume_dev_t)))
+	    		open_bdev(MAX_SWAPFILES, resume_swap_dev_t, 1)))
 		return 1;
-	
+
 	return 0;
 }
 
-static void swapwriter_cleanup(int ending_cycle)
+static void suspend_swap_cleanup(int ending_cycle)
 {
 	if (ending_cycle)
 		disable_swapfile();
@@ -448,7 +467,7 @@ static void swapwriter_cleanup(int ending_cycle)
 	close_bdevs();
 }
 
-static int swapwriter_release_storage(void)
+static int suspend_swap_release_storage(void)
 {
 	int i = 0;
 
@@ -457,17 +476,15 @@ static int swapwriter_release_storage(void)
 		return 0;
 
 	header_pages_allocated = 0;
+	main_pages_allocated = 0;
 
 	if (swapextents.first) {
 		/* Free swap entries */
 		struct extent *extentpointer;
 		unsigned long extentvalue;
-		swp_entry_t entry;
 		suspend_extent_for_each(&swapextents, extentpointer, 
-				extentvalue) {
-			entry = extent_val_to_swap_entry(extentvalue);
-			swap_free(entry);
-		}
+				extentvalue)
+			swap_free(extent_val_to_swap_entry(extentvalue));
 
 		suspend_put_extent_chain(&swapextents);
 
@@ -479,17 +496,41 @@ static int swapwriter_release_storage(void)
 	return 0;
 }
 
+static int suspend_swap_allocate_storage(int space_requested)
+{
+	if (!__suspend_swap_allocate_storage(space_requested,
+				header_pages_allocated)) {
+		main_pages_requested = space_requested;
+		return 0;
+	}
+
+	return -ENOSPC;
+}
+
+static void free_swap_range(unsigned long min, unsigned long max)
+{
+	int j;
+
+	for (j = min; j < max; j++)
+		swap_free(extent_val_to_swap_entry(j));
+}
+
 /* 
  * Round robin allocation (where swap storage has the same priority).
  * could make this very inefficient, so we track extents allocated on
  * a per-swapfiles basis.
  */
-static int swapwriter_allocate_storage(int space_requested)
+static int __suspend_swap_allocate_storage(int main_space_requested,
+		int header_space_requested)
 {
-	int i, result = 0, first[MAX_SWAPFILES];
-	int pages_to_get = space_requested - swapextents.size;
+	int i, result = 0, first[MAX_SWAPFILES], pages_to_get, extra_pages, gotten = 0;
 	unsigned long extent_min[MAX_SWAPFILES], extent_max[MAX_SWAPFILES];
 
+	extra_pages = DIV_ROUND_UP(main_space_requested * (sizeof(unsigned long)
+			       + sizeof(int)), PAGE_SIZE);
+	pages_to_get = main_space_requested + extra_pages +
+		header_space_requested - swapextents.size;
+
 	if (pages_to_get < 1)
 		return 0;
 
@@ -508,67 +549,65 @@ static int swapwriter_allocate_storage(int space_requested)
 		unsigned swapfilenum;
 
 		entry = get_swap_page();
-		if (!entry.val) {
-			printk("Failed to get a swap page.\n");
-			result = -ENOSPC;
+		if (!entry.val)
 			break;
-		}
 
 		swapfilenum = swp_type(entry);
 		new_value = swap_entry_to_extent_val(entry);
+
 		if (first[swapfilenum]) {
 			first[swapfilenum] = 0;
-			extent_min[swapfilenum] = extent_max[swapfilenum] =
-				new_value;
-		} else {
-			if (new_value == extent_max[swapfilenum] + 1)
-				extent_max[swapfilenum]++;
-			else {
-				suspend_add_to_extent_chain(
-					&swapextents,
+			extent_min[swapfilenum] = new_value;
+			extent_max[swapfilenum] = new_value;
+			gotten++;
+			continue;
+		}
+
+		if (new_value == extent_max[swapfilenum] + 1) {
+			extent_max[swapfilenum]++;
+			gotten++;
+			continue;
+		}
+
+		if (suspend_add_to_extent_chain(&swapextents,
 					extent_min[swapfilenum],
+					extent_max[swapfilenum])) {
+			free_swap_range(extent_min[swapfilenum],
 					extent_max[swapfilenum]);
-				extent_min[swapfilenum] =
-					extent_max[swapfilenum] = new_value;
-			}
+			swap_free(entry);
+			gotten -= (extent_max[swapfilenum] -
+					extent_min[swapfilenum]);
+			break;
+		} else {
+			extent_min[swapfilenum] = new_value;
+			extent_max[swapfilenum] = new_value;
+			gotten++;
 		}
 	}
 
 	for (i = 0; i < MAX_SWAPFILES; i++)
-		if (!first[i])
-			suspend_add_to_extent_chain(
-				&swapextents,
-				extent_min[i], extent_max[i]);
+		if (!first[i] && suspend_add_to_extent_chain(&swapextents,
+					extent_min[i], extent_max[i])) {
+			free_swap_range(extent_min[i], extent_max[i]);
+			gotten -= (extent_max[i] - extent_min[i]);
+		}
 
+	if (gotten < pages_to_get)
+		result = -ENOSPC;
+
+	main_pages_allocated += gotten;
 	get_main_pool_phys_params();
 	return result;
 }
 
-static int swapwriter_write_header_init(void)
+static int suspend_swap_write_header_init(void)
 {
 	int i, result;
 	struct swap_info_struct *si;
 
-	suspend_header_bytes_used = 0;
-
 	suspend_extent_state_goto_start(&suspend_writer_posn);
-	/* Forward one page will be done prior to the read */
 
-	for (i = 0; i < MAX_SWAPFILES; i++) {
-		si = get_swap_info_struct(i);
-		if (si->swap_file)
-			devinfo[i].dev_t = si->bdev->bd_dev;
-		else
-			devinfo[i].dev_t = (dev_t) 0;
-	}
-
-	suspend_writer_buffer = (char *) get_zeroed_page(GFP_ATOMIC);
-	if (!suspend_writer_buffer) {
-		printk("Failed to get swapwriter buffer.\n");
-		return -ENOMEM;
-	}
-
-	suspend_writer_buffer_posn = 0;
+	suspend_writer_buffer_posn = suspend_header_bytes_used = 0;
 
 	/* Info needed to bootstrap goes at the start of the header.
 	 * First we save the positions and devinfo, including the number
@@ -579,24 +618,33 @@ static int swapwriter_write_header_init(void)
 	 * next header page by the time we go to use it.
 	 */
 
+	/* Forward one page will be done prior to the read */
+	for (i = 0; i < MAX_SWAPFILES; i++) {
+		si = get_swap_info_struct(i);
+		if (si->swap_file)
+			devinfo[i].dev_t = si->bdev->bd_dev;
+		else
+			devinfo[i].dev_t = (dev_t) 0;
+	}
+
 	if ((result = suspend_bio_ops.rw_header_chunk(WRITE,
-			&swapwriterops,
+			&suspend_swapops,
 			(char *) &suspend_writer_posn_save, 
 			sizeof(suspend_writer_posn_save))))
 		return result;
 
 	if ((result = suspend_bio_ops.rw_header_chunk(WRITE,
-			&swapwriterops,
+			&suspend_swapops,
 			(char *) &devinfo, sizeof(devinfo))))
 		return result;
 
 	for (i=0; i < MAX_SWAPFILES; i++)
-		suspend_serialise_extent_chain(&swapwriterops, &block_chain[i]);
+		suspend_serialise_extent_chain(&suspend_swapops, &block_chain[i]);
 
 	return 0;
 }
 
-static int swapwriter_write_header_cleanup(void)
+static int suspend_swap_write_header_cleanup(void)
 {
 	int result;
 	struct swap_info_struct *si;
@@ -625,9 +673,6 @@ static int swapwriter_write_header_cleanup(void)
 			resume_firstblock,
 			virt_to_page(suspend_writer_buffer));
 
-	free_page((unsigned long) suspend_writer_buffer);
-	suspend_writer_buffer = NULL;
-	
 	suspend_bio_ops.finish_all_io();
 
 	return result;
@@ -642,7 +687,7 @@ static int swapwriter_write_header_cleanup(void)
  * 1. Attempt to read the device specified with resume2=.
  * 2. Check the contents of the swap header for our signature.
  * 3. Warn, ignore, reset and/or continue as appropriate.
- * 4. If continuing, read the swapwriter configuration section
+ * 4. If continuing, read the suspend_swap configuration section
  *    of the header and set up block device info so we can read
  *    the rest of the header & image.
  *
@@ -652,19 +697,12 @@ static int swapwriter_write_header_cleanup(void)
  * normally.
  */
 
-static int swapwriter_read_header_init(void)
+static int suspend_swap_read_header_init(void)
 {
-	int i;
-
-	BUG_ON(!resume_block_device);
-	BUG_ON(!resume_dev_t);
+	int i, result = 0;
 
 	suspend_header_bytes_used = 0;
 
-	suspend_writer_buffer = (char *) get_zeroed_page(GFP_ATOMIC);
-
-	BUG_ON(!suspend_writer_buffer);
-
 	if (!header_dev_t) {
 		printk("read_header_init called when we haven't "
 				"verified there is an image!\n");
@@ -672,11 +710,11 @@ static int swapwriter_read_header_init(void)
 	}
 
 	/* 
-	 * If the header is not on the resume_dev_t, get the resume device first.
+	 * If the header is not on the resume_swap_dev_t, get the resume device first.
 	 */
-	if (header_dev_t != resume_dev_t) {
+	if (header_dev_t != resume_swap_dev_t) {
 		header_block_device = open_bdev(MAX_SWAPFILES + 1,
-				header_dev_t);
+				header_dev_t, 1);
 
 		if (IS_ERR(header_block_device))
 			return PTR_ERR(header_block_device);
@@ -684,7 +722,7 @@ static int swapwriter_read_header_init(void)
 		header_block_device = resume_block_device;
 
 	/* 
-	 * Read swapwriter configuration.
+	 * Read suspend_swap configuration.
 	 * Headerblock size taken into account already.
 	 */
 	suspend_bio_ops.bdev_page_io(READ, header_block_device,
@@ -711,10 +749,9 @@ static int swapwriter_read_header_init(void)
 		if (!thisdevice)
 			continue;
 
-		if (thisdevice == resume_dev_t) {
+		if (thisdevice == resume_swap_dev_t) {
 			devinfo[i].bdev = resume_block_device;
 			bdev_info_list[i] = bdev_info_list[MAX_SWAPFILES];
-			BUG_ON(!bdev_info_list[i]);
 			bdev_info_list[MAX_SWAPFILES] = NULL;
 			continue;
 		}
@@ -722,37 +759,35 @@ static int swapwriter_read_header_init(void)
 		if (thisdevice == header_dev_t) {
 			devinfo[i].bdev = header_block_device;
 			bdev_info_list[i] = bdev_info_list[MAX_SWAPFILES + 1];
-			BUG_ON(!bdev_info_list[i]);
 			bdev_info_list[MAX_SWAPFILES + 1] = NULL;
 			continue;
 		}
 
-		result = open_bdev(i, thisdevice);
-		if (IS_ERR(result)) {
-			close_bdevs();
+		result = open_bdev(i, thisdevice, 1);
+		if (IS_ERR(result))
 			return PTR_ERR(result);
-		}
 	}
 
+	suspend_bio_ops.read_header_init();
 	suspend_extent_state_goto_start(&suspend_writer_posn);
 	suspend_bio_ops.set_extra_page_forward();
 
-	for (i = 0; i < MAX_SWAPFILES; i++)
-		suspend_load_extent_chain(&block_chain[i]);
+	for (i = 0; i < MAX_SWAPFILES && !result; i++)
+		result = suspend_load_extent_chain(&block_chain[i]);
 
-	return 0;
+	return result;
 }
 
-static int swapwriter_read_header_cleanup(void)
+static int suspend_swap_read_header_cleanup(void)
 {
-	free_page((unsigned long) suspend_writer_buffer);
+	suspend_bio_ops.rw_cleanup(READ);
 	return 0;
 }
 
-/* swapwriter_invalidate_image
+/* suspend_swap_invalidate_image
  * 
  */
-static int swapwriter_invalidate_image(void)
+static int suspend_swap_invalidate_image(void)
 {
 	union p_diskpage cur;
 	int result = 0;
@@ -770,7 +805,7 @@ static int swapwriter_invalidate_image(void)
 	 */
 
 	if (nr_suspends > 0)
-		swapwriter_release_storage();
+		suspend_swap_release_storage();
 
 	/* 
 	 * We don't do a sanity check here: we want to restore the swap 
@@ -797,7 +832,7 @@ static int swapwriter_invalidate_image(void)
 			virt_to_page(cur.pointer));
 
 	if (!nr_suspends)
-		printk(KERN_WARNING name_suspend "Image invalidated.\n");
+		printk(KERN_WARNING "Suspend2: Image invalidated.\n");
 out:
 	suspend_bio_ops.finish_all_io();
 	free_page(cur.address);
@@ -813,7 +848,7 @@ out:
  * we have enough memory to be able to suspend & resume).
  *
  */
-static unsigned long swapwriter_memory_needed(void)
+static int suspend_swap_memory_needed(void)
 {
 	return 1;
 }
@@ -823,17 +858,17 @@ static unsigned long swapwriter_memory_needed(void)
  *
  * Description:
  */
-static int swapwriter_print_debug_stats(char *buffer, int size)
+static int suspend_swap_print_debug_stats(char *buffer, int size)
 {
 	int len = 0;
 	struct sysinfo sysinfo;
 	
-	if (suspend_active_writer != &swapwriterops) {
-		len = snprintf_used(buffer, size, "- Swapwriter inactive.\n");
+	if (suspendActiveAllocator != &suspend_swapops) {
+		len = snprintf_used(buffer, size, "- SwapAllocator inactive.\n");
 		return len;
 	}
 
-	len = snprintf_used(buffer, size, "- Swapwriter active.\n");
+	len = snprintf_used(buffer, size, "- SwapAllocator active.\n");
 	if (swapfilename[0])
 		len+= snprintf_used(buffer+len, size-len,
 			"  Attempting to automatically swapon: %s.\n", swapfilename);
@@ -841,7 +876,7 @@ static int swapwriter_print_debug_stats(char *buffer, int size)
 	si_swapinfo(&sysinfo);
 	
 	len+= snprintf_used(buffer+len, size-len, "  Swap available for image: %ld pages.\n",
-			sysinfo.freeswap + swapwriter_storage_allocated());
+			(int) sysinfo.freeswap + suspend_swap_storage_allocated());
 
 	return len;
 }
@@ -850,14 +885,14 @@ static int swapwriter_print_debug_stats(char *buffer, int size)
  * Storage needed
  *
  * Returns amount of space in the swap header required
- * for the swapwriter's data. This ignores the links between
+ * for the suspend_swap's data. This ignores the links between
  * pages, which we factor in when allocating the space.
  *
  * We ensure the space is allocated, but actually save the
  * data from write_header_init and therefore don't also define a
  * save_config_info routine.
  */
-static unsigned long swapwriter_storage_needed(void)
+static int suspend_swap_storage_needed(void)
 {
 	int i, result;
 	result = sizeof(suspend_writer_posn_save) + sizeof(devinfo);
@@ -865,7 +900,7 @@ static unsigned long swapwriter_storage_needed(void)
 	for (i = 0; i < MAX_SWAPFILES; i++) {
 		result += 3 * sizeof(int);
 		result += (2 * sizeof(unsigned long) * 
-			(block_chain[i].allocs - block_chain[i].frees));
+			block_chain[i].num_extents);
 	}
 
 	return result;
@@ -874,20 +909,21 @@ static unsigned long swapwriter_storage_needed(void)
 /*
  * Image_exists
  */
-static int swapwriter_image_exists(void)
+static int suspend_swap_image_exists(void)
 {
 	int signature_found;
 	union p_diskpage diskpage;
 	
-	if (!resume_dev_t) {
+	if (!resume_swap_dev_t) {
 		printk("Not even trying to read header "
-				"because resume_dev_t is not set.\n");
+				"because resume_swap_dev_t is not set.\n");
 		return 0;
 	}
 	
 	if (!resume_block_device &&
-	    IS_ERR(resume_block_device = open_bdev(MAX_SWAPFILES, resume_dev_t))) {
-		printk("Failed to open resume dev_t (%x).\n", resume_dev_t);
+	    IS_ERR(resume_block_device =
+			open_bdev(MAX_SWAPFILES, resume_swap_dev_t, 1))) {
+		printk("Failed to open resume dev_t (%x).\n", resume_swap_dev_t);
 		return 0;
 	}
 
@@ -902,22 +938,21 @@ static int swapwriter_image_exists(void)
 	free_page(diskpage.address);
 
 	if (signature_found < 2) {
-		printk(name_suspend "Normal swapspace found.\n");
+		printk("Suspend2: Normal swapspace found.\n");
 		return 0;	/* Normal swap space */
 	} else if (signature_found == -1) {
-		printk(KERN_ERR name_suspend
-			"Unable to find a signature. Could you have moved "
-			"a swap file?\n");
+		printk(KERN_ERR "Suspend2: Unable to find a signature. Could "
+				"you have moved a swap file?\n");
 		return 0;
 	} else if (signature_found < 6) {
-		printk(name_suspend "Dectected another implementation's signature.\n");
+		printk("Suspend2: Detected another implementation's signature.\n");
 		return 0;
 	} else if ((signature_found >> 1) != SIGNATURE_VER) {
 		if ((!(test_suspend_state(SUSPEND_NORESUME_SPECIFIED))) &&
 			suspend_early_boot_message(1, SUSPEND_CONTINUE_REQ,
 			 "Found a different style suspend image signature.")) {
 			set_suspend_state(SUSPEND_NORESUME_SPECIFIED);
-			printk(name_suspend "Dectected another implementation's signature.\n");
+			printk("Suspend2: Dectected another implementation's signature.\n");
 		}
 	}
 
@@ -929,14 +964,14 @@ static int swapwriter_image_exists(void)
  *
  * Record that we tried to resume from this image.
  */
-static void swapwriter_mark_resume_attempted(void)
+static void suspend_swap_mark_resume_attempted(int mark)
 {
 	union p_diskpage diskpage;
 	int signature_found;
 	
-	if (!resume_dev_t) {
+	if (!resume_swap_dev_t) {
 		printk("Not even trying to record attempt at resuming"
-				" because resume_dev_t is not set.\n");
+				" because resume_swap_dev_t is not set.\n");
 		return;
 	}
 	
@@ -950,7 +985,9 @@ static void swapwriter_mark_resume_attempted(void)
 	switch (signature_found) {
 		case 12:
 		case 13:
-			diskpage.pointer->swh.magic.magic[5] |= 0x80;
+			diskpage.pointer->swh.magic.magic[5] &= ~0x80;
+			if (mark)
+				diskpage.pointer->swh.magic.magic[5] |= 0x80;
 			break;
 	}
 	
@@ -959,8 +996,6 @@ static void swapwriter_mark_resume_attempted(void)
 			virt_to_page(diskpage.ptr));
 	suspend_bio_ops.finish_all_io();
 	free_page(diskpage.address);
-	
-	close_bdevs();
 	return;
 }
 
@@ -976,11 +1011,12 @@ static void swapwriter_mark_resume_attempted(void)
  * FIRSTBLOCK is the location of the first block in the swap file
  * (specifying for a swap partition is nonsensical but not prohibited).
  * Data is validated by attempting to read a swap header from the
- * location given. Failure will result in swapwriter refusing to
+ * location given. Failure will result in suspend_swap refusing to
  * save an image, and a reboot with correct parameters will be
  * necessary.
  */
-static int swapwriter_parse_sig_location(char *commandline, int only_writer)
+static int suspend_swap_parse_sig_location(char *commandline,
+		int only_allocator, int quiet)
 {
 	char *thischar, *devstart, *colon = NULL, *at_symbol = NULL;
 	union p_diskpage diskpage;
@@ -990,7 +1026,7 @@ static int swapwriter_parse_sig_location(char *commandline, int only_writer)
 		/* 
 		 * Failing swap:, we'll take a simple
 		 * resume2=/dev/hda2, but fall through to
-		 * other writers if /dev/ isn't matched.
+		 * other allocators if /dev/ isn't matched.
 		 */
 		if (strncmp(commandline, "/dev/", 5))
 			return 1;
@@ -1028,13 +1064,16 @@ static int swapwriter_parse_sig_location(char *commandline, int only_writer)
 	if (at_symbol) {
 		resume_blocksize = (int) simple_strtoul(at_symbol + 1, NULL, 0);
 		if (resume_blocksize & (SECTOR_SIZE - 1)) {
-			printk("Swapwriter: Blocksizes are multiples of %d!\n", SECTOR_SIZE);
+			if (!quiet)
+				printk("SwapAllocator: Blocksizes are multiples"
+						"of %d!\n", SECTOR_SIZE);
 			return -EINVAL;
 		}
-		resume_firstblock = resume_firstblock * (resume_blocksize / SECTOR_SIZE);
+		resume_firstblock = resume_firstblock *
+			(resume_blocksize / SECTOR_SIZE);
 	}
 	
-	temp_result = try_to_parse_resume_device(devstart);
+	temp_result = try_to_parse_resume_device(devstart, quiet);
 
 	if (colon)
 		*colon = ':';
@@ -1046,7 +1085,8 @@ static int swapwriter_parse_sig_location(char *commandline, int only_writer)
 
 	diskpage.address = get_zeroed_page(GFP_ATOMIC);
 	if (!diskpage.address) {
-		printk(KERN_ERR name_suspend "Swapwriter: Failed to allocate a diskpage for I/O.\n");
+		printk(KERN_ERR "Suspend2: SwapAllocator: Failed to allocate "
+					"a diskpage for I/O.\n");
 		return -ENOMEM;
 	}
 
@@ -1058,14 +1098,16 @@ static int swapwriter_parse_sig_location(char *commandline, int only_writer)
 	suspend_bio_ops.finish_all_io();
 	
 	if (temp_result) {
-		printk(KERN_ERR name_suspend "Swapwriter: Failed to submit I/O.\n");
+		printk(KERN_ERR "Suspend2: SwapAllocator: Failed to submit "
+					"I/O.\n");
 		goto invalid;
 	}
 	
 	signature_found = parse_signature(diskpage.pointer->swh.magic.magic, 0);
 
 	if (signature_found != -1) {
-		printk(name_suspend "Swapwriter: Signature found.\n");
+		if (!quiet)
+			printk("Suspend2: SwapAllocator: Signature found.\n");
 		result = 0;
 
 		suspend_bio_ops.set_devinfo(devinfo);
@@ -1074,7 +1116,9 @@ static int swapwriter_parse_sig_location(char *commandline, int only_writer)
 		set_suspend_state(SUSPEND_CAN_SUSPEND);
 		set_suspend_state(SUSPEND_CAN_RESUME);
 	} else
-		printk(KERN_ERR name_suspend "Swapwriter: No swap signature found at specified location.\n");
+		if (!quiet)
+			printk(KERN_ERR "Suspend2: SwapAllocator: No swap "
+				"signature found at specified location.\n");
 invalid:
 	free_page((unsigned long) diskpage.address);
 	return result;
@@ -1103,7 +1147,8 @@ static int header_locations_read_sysfs(const char *page, int count)
 			haveswap = 1;
 			if (!printedpartitionsmessage) {
 				len += sprintf(output + len, 
-					"For swap partitions, simply use the format: resume2=swap:/dev/hda1.\n");
+					"For swap partitions, simply use the "
+					"format: resume2=swap:/dev/hda1.\n");
 				printedpartitionsmessage = 1;
 			}
 		} else {
@@ -1119,111 +1164,99 @@ static int header_locations_read_sysfs(const char *page, int count)
 			swapf = si->swap_file->f_mapping->host;
 			if (!(zone = bmap(swapf,0))) {
 				len+= sprintf(output + len, 
-					"Swapfile %s has been corrupted. Reuse mkswap on it and try again.\n",
+					"Swapfile %s has been corrupted. Reuse"
+					" mkswap on it and try again.\n",
 					path_page);
 			} else {
 				char name_buffer[255];
-				len+= sprintf(output + len, "For swapfile `%s`, use resume2=swap:/dev/%s:0x%x.\n",
-						path_page,
-						bdevname(si->bdev, name_buffer),
-						zone << (swapf->i_blkbits - 9));
+				len+= sprintf(output + len, "For swapfile `%s`,"
+					" use resume2=swap:/dev/%s:0x%x.\n",
+					path_page,
+					bdevname(si->bdev, name_buffer),
+					zone << (swapf->i_blkbits - 9));
 			}
 
 		}
 	}
 	
 	if (!haveswap)
-		len = sprintf(output, "You need to turn on swap partitions before examining this file.\n");
+		len = sprintf(output, "You need to turn on swap partitions "
+				"before examining this file.\n");
 
 	free_page((unsigned long) path_page);
 	return len;
 }
 
-static struct suspend_sysfs_data swapwriter_sysfs_data[] = {
+static struct suspend_sysfs_data sysfs_params[] = {
 	{
 	 SUSPEND2_ATTR("swapfilename", SYSFS_RW),
-	 SYSFS_STRING(swapfilename, 255, SYSFS_SM_NOT_NEEDED)
+	 SYSFS_STRING(swapfilename, 255, 0)
 	},
 
 	{
 	 SUSPEND2_ATTR("headerlocations", SYSFS_READONLY),
-	 SYSFS_CUSTOM(header_locations_read_sysfs, NULL, SYSFS_SM_NOT_NEEDED)
+	 SYSFS_CUSTOM(header_locations_read_sysfs, NULL, 0)
 	},
 
 	{ SUSPEND2_ATTR("enabled", SYSFS_RW),
-	  SYSFS_INT(&swapwriterops.enabled, 0, 1),
+	  SYSFS_INT(&suspend_swapops.enabled, 0, 1, 0),
 	  .write_side_effect		= attempt_to_parse_resume_device2,
 	}
 };
 
-static struct suspend_module_ops swapwriterops = {
+static struct suspend_module_ops suspend_swapops = {
 	.type					= WRITER_MODULE,
-	.name					= "Swap Writer",
+	.name					= "Swap Allocator",
+	.directory				= "swap",
 	.module					= THIS_MODULE,
-	.memory_needed				= swapwriter_memory_needed,
-	.print_debug_info			= swapwriter_print_debug_stats,
-	.storage_needed				= swapwriter_storage_needed,
-	.initialise				= swapwriter_initialise,
-	.cleanup				= swapwriter_cleanup,
+	.memory_needed				= suspend_swap_memory_needed,
+	.print_debug_info			= suspend_swap_print_debug_stats,
+	.storage_needed				= suspend_swap_storage_needed,
+	.initialise				= suspend_swap_initialise,
+	.cleanup				= suspend_swap_cleanup,
 
-	.noresume_reset	= swapwriter_noresume_reset,
-	.storage_available 	= swapwriter_storage_available,
-	.storage_allocated	= swapwriter_storage_allocated,
-	.release_storage	= swapwriter_release_storage,
-	.allocate_header_space	= swapwriter_allocate_header_space,
-	.allocate_storage	= swapwriter_allocate_storage,
-	.image_exists		= swapwriter_image_exists,
-	.mark_resume_attempted	= swapwriter_mark_resume_attempted,
-	.write_header_init	= swapwriter_write_header_init,
-	.write_header_cleanup	= swapwriter_write_header_cleanup,
-	.read_header_init	= swapwriter_read_header_init,
-	.read_header_cleanup	= swapwriter_read_header_cleanup,
-	.invalidate_image	= swapwriter_invalidate_image,
-	.parse_sig_location	= swapwriter_parse_sig_location,
+	.noresume_reset	= suspend_swap_noresume_reset,
+	.storage_available 	= suspend_swap_storage_available,
+	.storage_allocated	= suspend_swap_storage_allocated,
+	.release_storage	= suspend_swap_release_storage,
+	.allocate_header_space	= suspend_swap_allocate_header_space,
+	.allocate_storage	= suspend_swap_allocate_storage,
+	.image_exists		= suspend_swap_image_exists,
+	.mark_resume_attempted	= suspend_swap_mark_resume_attempted,
+	.write_header_init	= suspend_swap_write_header_init,
+	.write_header_cleanup	= suspend_swap_write_header_cleanup,
+	.read_header_init	= suspend_swap_read_header_init,
+	.read_header_cleanup	= suspend_swap_read_header_cleanup,
+	.invalidate_image	= suspend_swap_invalidate_image,
+	.parse_sig_location	= suspend_swap_parse_sig_location,
+
+	.sysfs_data		= sysfs_params,
+	.num_sysfs_entries	= sizeof(sysfs_params) / sizeof(struct suspend_sysfs_data),
 };
 
 /* ---- Registration ---- */
-static __init int swapwriter_load(void)
+static __init int suspend_swap_load(void)
 {
-	int result;
-	int i, numfiles = sizeof(swapwriter_sysfs_data) / sizeof(struct suspend_sysfs_data);
-	
-	printk("Suspend2 Swap Writer loading.\n");
+	suspend_swapops.rw_init = suspend_bio_ops.rw_init;
+	suspend_swapops.rw_cleanup = suspend_bio_ops.rw_cleanup;
+	suspend_swapops.read_chunk = suspend_bio_ops.read_chunk;
+	suspend_swapops.write_chunk = suspend_bio_ops.write_chunk;
+	suspend_swapops.rw_header_chunk = suspend_bio_ops.rw_header_chunk;
 
-	swapwriterops.rw_init = suspend_bio_ops.rw_init;
-	swapwriterops.rw_cleanup = suspend_bio_ops.rw_cleanup;
-	swapwriterops.read_chunk = suspend_bio_ops.read_chunk;
-	swapwriterops.write_chunk = suspend_bio_ops.write_chunk;
-	swapwriterops.rw_header_chunk = suspend_bio_ops.rw_header_chunk;
-
-	if (!(result = suspend_register_module(&swapwriterops))) {
-		struct kobject *kobj = make_suspend2_sysdir("swapwriter");
-		for (i=0; i< numfiles; i++)
-			suspend_register_sysfs_file(kobj,
-					&swapwriter_sysfs_data[i]);
-	} else
-		printk("Suspend2 Swap Writer unable to register!\n");
-	return result;
+	return suspend_register_module(&suspend_swapops);
 }
 
 #ifdef MODULE
-static __exit void swapwriter_unload(void)
+static __exit void suspend_swap_unload(void)
 {
-	int i, numfiles = sizeof(swapwriter_sysfs_data) /
-		    sizeof(struct suspend_sysfs_data);
-
-	printk("Suspend2 Swap Writer unloading.\n");
-
-	for (i=0; i< numfiles; i++)
-		suspend_unregister_sysfs_file(&swapwriter_sysfs_data[i]);
-	suspend_unregister_module(&swapwriterops);
+	suspend_unregister_module(&suspend_swapops);
 }
 
-module_init(swapwriter_load);
-module_exit(swapwriter_unload);
+module_init(suspend_swap_load);
+module_exit(suspend_swap_unload);
 MODULE_LICENSE("GPL");
 MODULE_AUTHOR("Nigel Cunningham");
-MODULE_DESCRIPTION("Suspend2 swap writer");
+MODULE_DESCRIPTION("Suspend2 SwapAllocator");
 #else
-late_initcall(swapwriter_load);
+late_initcall(suspend_swap_load);
 #endif
